/* iSCSI driver for Linux
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsi-session.h,v 1.1.2.4 2004/08/12 08:46:52 krishmnc Exp $
 *
 * iscsi-session.h
 *
 *  define the user level iSCSI session structure needed by the login library
 */

#ifndef ISCSI_SESSION_H_
#define ISCSI_SESSION_H_

#include "iscsi-auth-client.h"
#include "iscsi-common.h"

/* daemon's session structure */
struct iscsi_session {
	int socket_fd;
	int login_timeout;
	int auth_timeout;
	int active_timeout;
	int idle_timeout;
	int ping_timeout;
	int vendor_specific_keys;
	unsigned int irrelevant_keys_bitmap;
	int send_async_text;
	uint32_t itt;
	uint32_t cmd_sn;
	uint32_t exp_cmd_sn;
	uint32_t max_cmd_sn;
	uint32_t exp_stat_sn;
	int immediate_data;
	int initial_r2t;
	int max_recv_data_segment_len;	/* the value we declare */
	int max_xmit_data_segment_len;	/* the value declared by the target */
	int first_burst_len;
	int max_burst_len;
	int data_pdu_in_order;
	int data_seq_in_order;
	int def_time2wait;
	int def_time2retain;
	int header_digest;
	int data_digest;
	int type;
	int current_stage;
	int next_stage;
	int partial_response;
	int portal_group_tag;
	uint8_t isid[6];
	uint16_t tsid;
	int channel;
	int target_id;
	char target_name[TARGET_NAME_MAXLEN + 1];
	char target_alias[TARGET_NAME_MAXLEN + 1];
	char *initiator_name;
	char *initiator_alias;
	int ip_length;
	uint8_t ip_address[16];
	int port;
	int tcp_window_size;
	int (*update_address) (struct iscsi_session * session, char *address);
	struct auth_str_block auth_recv_string_block;
	struct auth_str_block auth_send_string_block;
	struct auth_large_binary auth_recv_binary_block;
	struct auth_large_binary auth_send_binary_block;
	struct iscsi_acl auth_client_block;
	struct iscsi_acl *auth_client;
	int num_auth_buffers;
	struct auth_buffer_desc auth_buffers[5];
	int bidirectional_auth;
	char username[AUTH_STR_MAX_LEN];
	uint8_t password[AUTH_STR_MAX_LEN];
	int password_length;
	char username_in[AUTH_STR_MAX_LEN];
	uint8_t password_in[AUTH_STR_MAX_LEN];
	int password_length_in;
};

#endif
