/*
 * iSCSI driver for Linux
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsi-ioctl.h,v 1.1.2.17 2004/09/14 21:19:45 mikenc Exp $
 *
 * include for ioctl calls between the daemon and the kernel module
 *
 */

#ifndef ISCSI_IOCTL_H_
#define ISCSI_IOCTL_H_

#include <linux/ioctl.h>
#include "iscsi-protocol.h"
#include "iscsi-portal.h"

struct iscsi_session_ioctl {
	uint32_t ioctl_size;
	uint32_t ioctl_version;
	uint32_t config_number;
	int update;
	uint8_t isid[6];
	int password_length;
	char username[AUTH_STR_MAX_LEN];
	unsigned char password[AUTH_STR_MAX_LEN];
	int password_length_in;
	char username_in[AUTH_STR_MAX_LEN];
	unsigned char password_in[AUTH_STR_MAX_LEN];
	unsigned char target_name[TARGET_NAME_MAXLEN + 1];
	unsigned char initiator_name[TARGET_NAME_MAXLEN + 1];
	unsigned char initiator_alias[TARGET_NAME_MAXLEN + 1];
	/* returned from the kernel */
	int host_number;
	int portal_failover;
	/*
	 * ipv6 fixups todo (only set bitmap fns use these now), but if
	 * failover is killed then the fixup is simple.
	 * Or this could be done in userspace too.
	 */
	char preferred_portal[16];
	char preferred_subnet[16];
	uint32_t preferred_subnet_mask;
	uint32_t num_portals;
	uint32_t portal_info_size;
	struct iscsi_portal_info portals[1];
	/* 1 or more portals for this session to use */
};

#define ISCSI_SESSION_IOCTL_VERSION 23

struct iscsi_portal_list {
	int host_no;
	struct iscsi_portal_info portals[MAX_PORTALS];
};

struct iscsi_ls_session {
	uint8_t isid[6];
	uint16_t tsid;
	int initial_r2t;
	int immediate_data;
	int header_digest;
	int data_digest;
	int first_burst_len;
	int max_burst_len;
	int max_recv_data_segment_len;
	int max_xmit_data_segment_len;
	int login_timeout;
	int auth_timeout;
	int active_timeout;
	int idle_timeout;
	int ping_timeout;
	time_t establishment_time;
	time_t session_drop_time;
	int ever_established;
	int session_alive;
	struct sockaddr addr;
};

struct target_info {
	unsigned char target_name[TARGET_NAME_MAXLEN + 1];
	unsigned char target_alias[TARGET_NAME_MAXLEN + 1];
	int host_no;
	int num_portals;
	struct iscsi_ls_session session_data;
};

/*
 * ioctls
 */
enum {
	ISCSI_EST_SESS_CMD,
	ISCSI_TGT_INFO_CMD,
	ISCSI_PRTL_INFO_CMD,
};

#define ISCSI_MAX_CMD_NR ISCSI_PRTL_INFO_CMD

#define ISCSI_IOCTL 0xbc
#define ISCSI_ESTABLISH_SESSION _IOWR(ISCSI_IOCTL, ISCSI_EST_SESS_CMD, \
					struct iscsi_session_ioctl)
#define ISCSI_LS_TARGET_INFO _IOWR(ISCSI_IOCTL, ISCSI_TGT_INFO_CMD, \
					struct target_info)
#define ISCSI_LS_PORTAL_INFO _IOWR(ISCSI_IOCTL, ISCSI_PRTL_INFO_CMD, \
					struct iscsi_portal_list)

#endif
