/*
 * iSCSI driver for Linux
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsi-portal.h,v 1.1.2.6 2004/09/14 21:13:04 mikenc Exp $
 *
 * portal info structure used in ioctls and the kernel module
 *
 */

#ifndef ISCSI_PORTAL_H_
#define ISCSI_PORTAL_H_

#include "iscsi-session.h"

struct iscsi_portal_info {
	int login_timeout;
	int auth_timeout;
	int active_timeout;
	int idle_timeout;
	int ping_timeout;
	int abort_timeout;
	int reset_timeout;
	int replacement_timeout;	/* FIXME: should this be per-session 
					 * rather than per-portal? 
					 */
	unsigned int disk_command_timeout;/* FIXME: should this be per-session 
					   * rather than per-portal? 
					   */
	int initial_r2t;
	int immediate_data;
	int max_recv_data_segment_len;
	int first_burst_len;
	int max_burst_len;
	int def_time2wait;
	int def_time2retain;
	int header_digest;
	int data_digest;
	int tag;
	int tcp_window_size;
	int type_of_service;
	struct sockaddr addr;
};

#define MAX_PORTALS 32		/* 
				 * 32 is the sizeof(unsigned int).
				 * If max portals to a target exceeds 32 then
				 * we need to change the 
				 * preferred_portal_bitmap,
				 * preferred_subnet_bitmap from unsigned int to
				 * an array of unsigned int's.
				 */

extern unsigned int find_portal(struct iscsi_session *session,
				struct sockaddr *addr);
extern void set_portal_config(struct iscsi_session *session, unsigned int p);
extern int set_portal(struct iscsi_session *session, unsigned int p);
extern void set_preferred_subnet(struct iscsi_session *session,
				 char *preferred_subnet, uint32_t subnet_mask);
extern void set_preferred_portal(struct iscsi_session *session,
				 char *preferred_portal);
extern int get_appropriate_portal(struct iscsi_session *session);
extern void next_portal(struct iscsi_session *session);
extern int same_network_portal(struct iscsi_portal_info *p1,
			       struct iscsi_portal_info *p2);
#endif
